@@ -24,6 +24,11 @@
 
 namespace antpm{
 
+AntChannel::AntChannel(const uchar ch)
+  : chan(ch)
+{
+}
+
 void
 AntChannel::addMsgListener2(AntListenerBase* lb)
 {
@@ -44,7 +49,8 @@ AntChannel::onMsg(AntMessage &m)
   boost::unique_lock<boost::mutex> lock(m_mtxListeners);
   for(std::list<AntListenerBase*>::iterator i = listeners.begin(); i != listeners.end(); i++)
   {
-    (*i)->onMsg(m);
+    AntListenerBase* listener = *i;
+    listener->onMsg(m);
   }
 }
 
@@ -54,10 +60,28 @@ AntChannel::interruptWait()
   boost::unique_lock<boost::mutex> lock(m_mtxListeners);
   for(std::list<AntListenerBase*>::iterator i = listeners.begin(); i != listeners.end(); i++)
   {
-    (*i)->interruptWait();
+    AntListenerBase* listener = *i;
+    listener->interruptWait();
   }
 }
 
+void
+AntChannel::sanityCheck(const char* caller)
+{
+  boost::unique_lock<boost::mutex> lock(m_mtxListeners);
+  if(!listeners.empty())
+  {
+    lprintf(LOG_DBG3, "sanityCheck[ch=%d] found %d leftover listeners (caller=%s)\n", static_cast<int>(chan), static_cast<int>(listeners.size()), caller);
+    for(size_t i = 0; i < listeners.size(); i++)
+    {
+      std::list<AntListenerBase*>::iterator iter = listeners.begin();
+      std::advance(iter, i);
+      AntListenerBase* listener = *iter;
+      std::string s = listener->name();
+      lprintf(LOG_DBG3, "sanityCheck[ch=%d] found i=%zu %s\n", static_cast<int>(chan), i, s.c_str());
+    }
+  }
+}
 
 
 
@@ -72,6 +96,8 @@ AntListenerBase::AntListenerBase(AntChannel& o)
 
 AntListenerBase::~AntListenerBase()
 {
+  //FIXME: what happens if this dtor is called while an other thread is still executing waitForMsg() ?
+  //boost::unique_lock<boost::mutex> lock(m_mtxResp);
   owner.rmMsgListener2(this);
 }
 
@@ -137,7 +163,7 @@ bool
 AntEvListener::match(AntMessage& other) const
 {
   return other.getMsgId()==MESG_RESPONSE_EVENT_ID
-      && owner.chan == other.getPayloadRef()[0]
+      && owner.getChan() == other.getPayloadRef()[0]
       && other.getPayloadRef()[1]==MESG_EVENT_ID
       && other.getPayloadRef()[2]!=EVENT_TRANSFER_TX_START; // let us not consider EVENT_TRANSFER_TX_START events
 }
@@ -163,7 +189,7 @@ bool
 AntRespListener::match(AntMessage& other) const
 {
   return other.getMsgId()==MESG_RESPONSE_EVENT_ID
-      && owner.chan == other.getPayloadRef()[0]
+      && owner.getChan() == other.getPayloadRef()[0]
       && other.getPayloadRef()[1]==msgId;
 }
 
@@ -205,8 +231,8 @@ bool
 AntBCastListener::match(AntMessage& other) const
 {
   return other.getMsgId()==MESG_BROADCAST_DATA_ID
-      && other.getPayloadRef()[0]==owner.chan
-      && other.getPayloadRef()[1]==first;
+      && other.getPayloadRef()[0]==owner.getChan()
+      && (other.getPayloadRef()[1]==ANTFS_BeaconId || other.getPayloadRef()[1]==ANTFS_CommandResponseId);
 }
 
 bool
@@ -235,18 +261,27 @@ AntBurstListener::onMsg(AntMessage& m)
   boost::unique_lock<boost::mutex> lock(m_mtxResp);
   if(match(m))
   {
-    bursts.push_back(m);
+    m_bursts.push_back(m);
     m_cndResp.notify_all();
   }
 }
 
+void
+AntBurstListener::interruptWait()
+{
+  boost::unique_lock<boost::mutex> lock(m_mtxResp);
+  m_bursts.clear();
+  m_cndResp.notify_all(); // intentionally generate a "spurious" wakeup
+}
+
+
 
 bool
 AntBurstListener::match(AntMessage& other) const
 {
   const M_ANT_Burst* burst(reinterpret_cast<const M_ANT_Burst*>(other.getPayloadRef()));
   return other.getMsgId()==MESG_BURST_DATA_ID
-    && burst->chan==owner.chan;
+    && burst->chan==owner.getChan();
 }
 
 
@@ -255,9 +290,9 @@ AntBurstListener::waitForBursts(std::list<AntMessage>& bs, const size_t timeout_
 {
   bs.clear();
   boost::unique_lock<boost::mutex> lock(m_mtxResp);
-  if(!bursts.empty()) // some had already arrived
+  if(!m_bursts.empty()) // some had already arrived
   {
-    std::swap(bs, bursts);
+    std::swap(bs, m_bursts);
     return true;
   }
   // TODO: handle arrival of event:EVENT_RX_FAIL
@@ -267,10 +302,10 @@ AntBurstListener::waitForBursts(std::list<AntMessage>& bs, const size_t timeout_
     return false;
   }
   // true means, either notification OR spurious wakeup!!
-  //assert(!bursts.empty()); // this might fail for spurious wakeups!!
-  if(!bursts.empty())
+  //assert(!m_bursts.empty()); // this might fail for spurious wakeups!!
+  if(!m_bursts.empty())
   {
-    std::swap(bs, bursts);
+    std::swap(bs, m_bursts);
     return true;
   }
   return false;
