@@ -30,6 +30,7 @@
 #include <algorithm>
 #include <vector>
 #include <string>
+#include <chrono>
 #include <boost/thread/thread_time.hpp>
 #include <iostream>
 #include "stdintfwd.hpp"
@@ -470,6 +471,7 @@ AntFr310XT::handleEvents()
   }
   else if(state == ST_ANTFS_AUTH1_PASS)
   {
+    bool is_fr405 = clientDevName=="Forerunner 405" || clientDevName=="Forerunner 410" || isAntpm405Override();
     CHECK_RETURN_FALSE_LOG_OK_DBG2(m_antMessenger->ANT_RequestMessage(chan, MESG_CHANNEL_STATUS_ID));
 
     if(!m_antMessenger->ANTFS_Authenticate(chan, hostSN, pairedKey))
@@ -483,7 +485,7 @@ AntFr310XT::handleEvents()
     // channel status <>
     CHECK_RETURN_FALSE_LOG_OK_DBG2(m_antMessenger->ANT_RequestMessage(chan, MESG_CHANNEL_STATUS_ID));
 
-    if(clientDevName=="Forerunner 405" || clientDevName=="Forerunner 410" || isAntpm405Override())
+    if(is_fr405)
       changeStateSafe(ST_ANTFS_GINTF_DL_CAPS);
     else if(mode==MD_DOWNLOAD_ALL || mode==MD_DIRECTORY_LISTING)
       changeStateSafe(ST_ANTFS_DL_DIRECTORY);
@@ -496,6 +498,8 @@ AntFr310XT::handleEvents()
   }
   else if(state == ST_ANTFS_DL_DIRECTORY)
   {
+    Database db = m_ds->getDatabaseFiles();
+
     CHECK_RETURN_FALSE(createDownloadFolder());
 
     //ANTFS_Upload(); //command pipe
@@ -518,6 +522,11 @@ AntFr310XT::handleEvents()
     LOG_VAR(zfc.activityFiles.size());
     LOG_VAR(zfc.courseFiles.size());
     LOG_VAR(zfc.waypointsFiles.size());
+    // zfc.cullFitFiles(db); // this culling strategy does not work well, because fit files are given new indices on the device after a download
+    zfc.cullFitFilesDate();
+    LOG_VAR(zfc.activityFiles.size());
+    LOG_VAR(zfc.courseFiles.size());
+    LOG_VAR(zfc.waypointsFiles.size());
 
     // TODO: read bcast here?
 
@@ -606,13 +615,19 @@ AntFr310XT::handleEvents()
                << "@" << DeviceSettings::time2str(t) << " newer than "
                << DeviceSettings::time2str(m_ds->LastTransferredTime) <<  "\n";
 
+      auto start = std::chrono::system_clock::now();
+
       std::vector<uchar> data;
       if(!m_antMessenger->ANTFS_Download(chan, fileIdx, data))
       {
         changeStateSafe(ST_ANTFS_LAST);
         return true;
       }
-      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << "\n\n\n";
+
+      auto end = std::chrono::system_clock::now();
+      auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
+      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << " (" << elapsed.count() << " ms, "
+                   << static_cast<double>(data.size())/static_cast<double>(elapsed.count())*1000.0 << " bps)\n\n\n";
       AntFsFile file0; file0.bytes=data; file0.saveToFile((folder+toString(fileIdx, 4, '0')+".fit").c_str());
       //LOG_VAR(file0.checkCrc());
 
@@ -644,13 +659,17 @@ AntFr310XT::handleEvents()
                << "@" << DeviceSettings::time2str(t) << " newer than "
                << DeviceSettings::time2str(m_ds->LastTransferredTime) <<  "\n";
 
+      auto start = std::chrono::system_clock::now();
       std::vector<uchar> data;
       if(!m_antMessenger->ANTFS_Download(chan, fileIdx, data))
       {
         changeStateSafe(ST_ANTFS_LAST);
         return true;
       }
-      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << "\n\n\n";
+      auto end = std::chrono::system_clock::now();
+      auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
+      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << " (" << elapsed.count() << " ms, "
+                   << static_cast<double>(data.size())/static_cast<double>(elapsed.count())*1000.0 << " bps)\n\n\n";
       AntFsFile file0; file0.bytes=data; file0.saveToFile((folder+toString(fileIdx, 4, '0')+".fit").c_str());
 
       fit.parse(data, gpx);
@@ -689,13 +708,17 @@ AntFr310XT::handleEvents()
                << "@" << DeviceSettings::time2str(t) << " newer than "
                << DeviceSettings::time2str(m_ds->LastTransferredTime) <<  "\n";
 
+      auto start = std::chrono::system_clock::now();
       std::vector<uchar> data;
       if(!m_antMessenger->ANTFS_Download(chan, fileIdx, data))
       {
         changeStateSafe(ST_ANTFS_LAST);
         return true;
       }
-      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << "\n\n\n";
+      auto end = std::chrono::system_clock::now();
+      auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(end - start);
+      LOG(LOG_RAW) << "\n\nDownloaded file idx=" << toString<ushort>(fileIdx,4,'0') << " (" << elapsed.count() << " ms, "
+                   << static_cast<double>(data.size())/static_cast<double>(elapsed.count())*1000.0 << " bps)\n\n\n";
       AntFsFile file0; file0.bytes=data; file0.saveToFile((folder+toString(fileIdx, 4, '0')+".fit").c_str());
 
       fit.parse(data, gpx);
