@@ -27,6 +27,7 @@
 #include <vector>
 #include <string>
 #include <iostream>
+#include <chrono>
 #include "common.hpp"
 #include <boost/thread/thread_time.hpp>
 #include <boost/foreach.hpp>
@@ -70,7 +71,10 @@ AntMessenger::AntMessenger()
   m_rpackQueue2.setOnDataArrivedCallback(std::bind1st(std::mem_fun(&AntMessenger::onMessage), this));
 
   for(int i=0; i < ANTPM_MAX_CHANNELS; i++)
-    chs[i].chan = i;
+  {
+    chs.push_back(std::make_unique<AntChannel>(i));
+    //chs[i].chan = i;
+  }
 
   packetIdx=0;
 }
@@ -176,7 +180,7 @@ AntMessenger::ANT_CloseChannel(uchar chan, const size_t timeout_ms)
   bool rv = sendCommand(m, timeout_ms/2);
   // TODO: read R[ 7] a4_03_40_00_01_07_e1                   MESG_RESPONSE_EVENT_ID chan=0x00 mId=MESG_EVENT_ID mCode=EVENT_CHANNEL_CLOSED
 
-  AntChannel& pc = chs[chan];
+  AntChannel& pc = *chs[chan].get();
   AntEvListener el(pc);
   //pc.addEvListener(&el);
 
@@ -344,7 +348,7 @@ AntMessenger::ANTFS_Pairing(const uchar chan, const uint hostSN, const std::stri
   cmd.detail.authenticate.sn      = hostSN;
   memcpy(cmd.name, &name[0], 8);
 
-  AntChannel& pc(chs[chan]);
+  AntChannel& pc = *chs[chan].get();
   AntBurstListener bl(pc);
   //pc.addMsgListener(&bl);
 
@@ -362,7 +366,7 @@ AntMessenger::ANTFS_Pairing(const uchar chan, const uint hostSN, const std::stri
     //AntMessage reply0;
     //waitForMessage(MESG_RESPONSE_EVENT_ID, &reply0, 2000);
 
-    AntChannel& pc(chs[chan]);
+    AntChannel& pc = *chs[chan].get();
     AntEvListener el(pc);
     //pc.addEvListener(&el);
 
@@ -445,7 +449,7 @@ AntMessenger::ANTFS_Authenticate(const uchar chan, const uint hostSN, const uint
     //AntMessage reply0;
     //waitForMessage(MESG_RESPONSE_EVENT_ID, &reply0, 2000);
 
-    AntChannel& pc(chs[chan]);
+    AntChannel& pc = *chs[chan].get();
     AntEvListener el(pc);
     //pc.addEvListener(&el);
 
@@ -562,6 +566,7 @@ AntMessenger::ANTFS_Download( const uchar chan, const ushort file, std::vector<u
   //R   3.109 52 MESG_CHANNEL_STATUS_ID chan=00 chanSt=Tracking
   CHECK_RETURN_FALSE(data.empty());
 
+  
   int errorCnt=0;
 
   //uint bytesReceived = 0;
@@ -583,12 +588,17 @@ AntMessenger::ANTFS_Download( const uchar chan, const ushort file, std::vector<u
     dl.crcSeed = crc;
     dl.maxBlockSize = 0;
 
-    AntChannel& pc(chs[chan]);
+    AntChannel& pc = *chs[chan].get();
+
+    std::vector<uchar> burstData;
+    {
     AntBurstListener bl(pc);
 
+    bool sentReqDl = false;
+
+    {
     AntEvListener el(pc);
 
-    bool sentReqDl = false;
     sentReqDl = false;
     //if(dlIter==0)
     //  CHECK_RETURN_FALSE_LOG_OK(collectBroadcasts(chan));
@@ -604,6 +614,7 @@ AntMessenger::ANTFS_Download( const uchar chan, const ushort file, std::vector<u
     sentReqDl = sentReqDl && el.waitForEvent(responseVal, 600);
     //pc.rmEvListener(&el);
     sentReqDl = sentReqDl && (responseVal==EVENT_TRANSFER_TX_COMPLETED);
+    }
 
     // TODO: handle event:EVENT_RX_FAIL and continue
 
@@ -615,13 +626,13 @@ AntMessenger::ANTFS_Download( const uchar chan, const ushort file, std::vector<u
 
     // TODO: read bcast here?
 
-    std::vector<uchar> burstData;
     if(!bl.collectBurst(burstData, 10000) && (++errorCnt<ANTPM_RETRIES))
     {
       //pc.rmMsgListener(&bl);
       continue;
     }
     //pc.rmMsgListener(&bl);
+    }
 
     //// ANTFS_RespDownload
     CHECK_RETURN_FALSE(burstData.size()>=3*8); // header and footer eats up 32 bytes already, but in case of error2 we only get 24 bytes
@@ -701,7 +712,7 @@ AntMessenger::ANTFS_Erase(const uchar chan, const ushort file)
 
   for(int itry=0; itry < ANTPM_RETRIES; itry++)
   {
-    AntChannel& pc(chs[chan]);
+    AntChannel& pc = *chs[chan].get();
     AntBurstListener bl(pc);
     //pc.addMsgListener(&bl);
 
@@ -748,12 +759,14 @@ AntMessenger::ANTFS_RequestClientDeviceSerialNumber(const uchar chan, const uint
   cmd.detail.authenticate.authStrLen = 0;
   cmd.detail.authenticate.sn = hostSN;
 
-  AntChannel& pc(chs[chan]);
+  CHECK_RETURN_FALSE_LOG_OK_DBG2(ANT_SendAcknowledgedData(chan, reinterpret_cast<uchar*>(&cmd), 2000));
+
+  {
+  AntChannel& pc = *chs[chan].get();
   AntBurstListener bl(pc);
   //pc.addMsgListener(&bl);
 
 
-  CHECK_RETURN_FALSE_LOG_OK_DBG2(ANT_SendAcknowledgedData(chan, reinterpret_cast<uchar*>(&cmd), 2000));
 
   std::vector<uchar> burstData;
   /*bool rv =*/ bl.collectBurst(burstData, 5000);
@@ -766,14 +779,23 @@ AntMessenger::ANTFS_RequestClientDeviceSerialNumber(const uchar chan, const uint
   //
   //CHECK_RETURN_FALSE_LOG_OK(waitForBurst(chan, burstData, 30000));
   LOG_VAR_DBG2(burstData.size());
-  CHECK_RETURN_FALSE_LOG_OK_DBG2(burstData.size()==4*8);
+  CHECK_RETURN_FALSE_LOG_OK_DBG2(burstData.size()==4*8 || burstData.size()==2*8);
 
   const M_ANTFS_Response* cmdResp(reinterpret_cast<const M_ANTFS_Response*>(&burstData[8]));
   sn = cmdResp->detail.authenticateResponse.sn;
   uchar lenDevName=cmdResp->detail.authenticateResponse.authStrLen; // 16 for 310XT, 14 for 410
-  CHECK_RETURN_FALSE_LOG_OK_DBG2(lenDevName>0);
+  LOG_VAR_DBG2(static_cast<int>(lenDevName));
+  if(burstData.size()==32)
+  {
+    CHECK_RETURN_FALSE_LOG_OK_DBG2(lenDevName>0);
 
-  devName = std::string(reinterpret_cast<const char*>(&burstData[16]), lenDevName);
+    devName = std::string(reinterpret_cast<const char*>(&burstData[16]), lenDevName);
+  }
+  else
+  {
+    devName = "Forerunner 405";
+  }
+  }
 
   logger() << "devName = \"" << devName << "\"\n";
 
@@ -808,7 +830,7 @@ AntMessenger::ANTFS_Direct(const uchar chan, const uint64_t code, std::vector<ui
     //AntMessage reply0;
     //waitForMessage(MESG_RESPONSE_EVENT_ID, &reply0, 2000);
 
-    AntChannel& pc(chs[chan]);
+    AntChannel& pc = *chs[chan].get();
     AntEvListener el(pc);
     //pc.addEvListener(&el);
 
@@ -892,7 +914,8 @@ AntMessenger::sendCommand(AntMessage &m, const size_t timeout_ms)
   }
 
   const uint8_t chan = m.getPayloadRef()[0];
-  AntChannel& pc = chs[chan];
+  AntChannel& pc = *chs[chan].get();
+  {
   AntRespListener respList(pc, m.getMsgId());
 
   assert(rv);
@@ -903,6 +926,9 @@ AntMessenger::sendCommand(AntMessage &m, const size_t timeout_ms)
     lprintf(antpm::LOG_ERR, "waitForResponse failed\n");
     return false;
   }
+  }
+
+  sanityCheck(__FUNCTION__);
 
   return true;
 }
@@ -913,11 +939,13 @@ AntMessenger::sendRequest(uchar reqMsgId, uchar chan, AntMessage *response, cons
 {
   AntMessage reqMsg(MESG_REQUEST_ID, chan, reqMsgId); // the request we're sending
 
-  AntChannel& pc = chs[chan];
+  AntChannel& pc = *chs[chan].get();
+  bool rv = false;
+  {
   AntReqListener rl(pc, reqMsgId, chan);
   //pc.addMsgListener(&rl);
 
-  bool rv = writeMessage(reqMsg);
+  rv = writeMessage(reqMsg);
   if(!rv)
   {
     lprintf(antpm::LOG_ERR, "writeMessage failed\n");
@@ -925,10 +953,17 @@ AntMessenger::sendRequest(uchar reqMsgId, uchar chan, AntMessage *response, cons
   }
 
 
-  AntMessage dummy;if(!response) response=&dummy;
+  AntMessage dummy;
+  if(!response)
+  {
+    response=&dummy;
+  }
+
   rv = rv&& rl.waitForMsg(response, timeout_ms);
 
   //pc.rmMsgListener(&rl);
+  }
+  sanityCheck(__FUNCTION__);
 
   return rv;
 }
@@ -979,11 +1014,13 @@ AntMessenger::sendAckData(const uchar chan, const uchar data[8], const size_t ti
   if(!m.assemble(mesg, buf, sizeof(buf)))
     return false;
 
-  AntChannel& pc(chs[chan]);
+  AntChannel& pc = *chs[chan].get();
+  bool rv = false;
+  {
   AntEvListener el(pc);
   //pc.addEvListener(&el);
 
-  bool rv = writeMessage(m);
+  rv = writeMessage(m);
   if(!rv)
   {
     lprintf(antpm::LOG_ERR, "writeMessage failed\n");
@@ -1003,6 +1040,8 @@ AntMessenger::sendAckData(const uchar chan, const uchar data[8], const size_t ti
       //lprintf(antpm::LOG_ERR, "no matching data ack before timeout\n"); fflush(stdout);
     }
     rv = rv && found;
+  }
+  sanityCheck(__FUNCTION__);
 
   return rv;
 }
@@ -1072,7 +1111,7 @@ AntMessenger::onMessage(std::vector<AntMessage> v)
        || m.getMsgId()==MESG_CHANNEL_STATUS_ID)
     {
       uint8_t chan=m.getPayloadRef()[0];
-      AntChannel& pc=chs[chan];
+      AntChannel& pc = *chs[chan].get();
       pc.onMsg(m);
     }
     else if(m.getMsgId()==MESG_BURST_DATA_ID)
@@ -1083,7 +1122,7 @@ AntMessenger::onMessage(std::vector<AntMessage> v)
       uint8_t chan=burst->chan;
       //chan = 0; // FIXME!!!
       //printf("burst? 0x%0x chan=%d\n", (int)m.getMsgId(), int(chan));
-      AntChannel& pc=chs[chan];
+      AntChannel& pc = *chs[chan].get();
       pc.onMsg(m);
     }
     else if(m.getMsgId()==MESG_STARTUP_MSG_ID)
@@ -1104,6 +1143,16 @@ AntMessenger::onMessage(std::vector<AntMessage> v)
 }
 
 
+void AntMessenger::sanityCheck(const char *caller)
+{
+  for(int i=0; i < ANTPM_MAX_CHANNELS; i++)
+  {
+    chs[i]->sanityCheck(caller);
+  }
+
+}
+
+
 bool
 AntMessenger::waitForBurst(const uchar chan,
                            std::vector<uchar>& burstData,
@@ -1117,7 +1166,8 @@ AntMessenger::waitForBurst(const uchar chan,
   bool found=false;
   bool lastFound=false;
 
-  AntChannel& pc(chs[chan]);
+  AntChannel& pc = *chs[chan].get();
+  {
   AntBurstListener bl(pc);
   //pc.addMsgListener(&bl);
   bool rv = bl.collectBurst(burstData, timeout_ms);
@@ -1128,6 +1178,9 @@ AntMessenger::waitForBurst(const uchar chan,
     lprintf(antpm::LOG_ERR, "couldn't reconstruct burst data transmission before timeout\n"); fflush(stdout);
     return false;
   }
+  }
+
+  sanityCheck(__FUNCTION__);
 
   return true;
 }
@@ -1136,11 +1189,11 @@ AntMessenger::waitForBurst(const uchar chan,
 bool
 AntMessenger::waitForBroadcast(const uchar chan, AntMessage* reply, const size_t timeout_ms)
 {
-  const uchar first=ANTFS_BeaconId;
   bool found=false;
 
-  AntChannel& pc(chs[chan]);
-  AntBCastListener bcl(pc, first);
+  AntChannel& pc = *chs[chan].get();
+  {
+  AntBCastListener bcl(pc);
   //pc.addBCastListener(&bcl);
 
   AntMessage dummy;if(!reply) reply=&dummy;
@@ -1149,8 +1202,13 @@ AntMessenger::waitForBroadcast(const uchar chan, AntMessage* reply, const size_t
   //M_ANTFS_Beacon* beacon(reinterpret_cast<M_ANTFS_Beacon*>(&reply->getPayloadRef()[1]));
   if(!found)
   {
-    lprintf(antpm::LOG_ERR, "no matching bcast before timeout\n"); fflush(stdout);
+    lprintf(antpm::LOG_ERR, "no matching bcast before timeout ch=%d\n", static_cast<int>(chan)); fflush(stdout);
   }
+
+  }
+
+  sanityCheck(__FUNCTION__);
+  
   return found;
 }
 
@@ -1160,8 +1218,10 @@ AntMessenger::interruptWait()
 {
   for(size_t i = 0; i < ANTPM_MAX_CHANNELS; i++)
   {
-    chs[i].interruptWait();
+    chs[i]->interruptWait();
   }
+
+  sanityCheck(__FUNCTION__);
 }
 
 
