@@ -29,7 +29,6 @@
 #include <signal.h>
 
 #include <functional>
-#include <tr1/functional>
 #include <algorithm>
 #include <vector>
 #include <string>
@@ -207,14 +206,14 @@ SerialTtyPrivate::guessDeviceName(vector<string> &guessedNames)
   else
   {
     LOG(LOG_DBG) << "Detecting in " << driverDir << " ...\n";
-    for(fs::recursive_directory_iterator end, iter(driverDir, fs::symlink_option::recurse); iter != end; ++iter)
+    for(fs::recursive_directory_iterator end, iter(driverDir, fs::symlink_option::recurse); iter != end; )
     {
       if(iter.level()>=2)
       {
         iter.pop();
         continue;
       }
-      //cout << dir.level() << ", " << *dir << std::endl;
+      //cout << iter.level() << ", " << *iter << std::endl;
       fs::path p = iter->path();
       //cout << p.parent_path() << "\n";
       if(iter.level()==1 && p.leaf()=="interface" && find_file_starts_with(p.parent_path(), "ttyUSB")!="")
@@ -230,6 +229,7 @@ SerialTtyPrivate::guessDeviceName(vector<string> &guessedNames)
         LOG(LOG_DBG) << "Found: \"" << iface << "\" as " << ttyUSB << " in " << p << "\n";
         guessedNames.push_back("/dev/"+ttyUSB);
       }
+      ++iter;
     }
   }
 //  // check for /sys/bus/usb-serial/drivers/cp210x/ttyUSBxxx
@@ -380,10 +380,10 @@ SerialTty::open()
     //
     // Log off and log on again for the changes to take effect!
     char se[256];
-    char* ss = strerror_r(m_p->m_fd, se, sizeof(se));
+    strerror_r(m_p->m_fd, se, sizeof(se));
     LOG(antpm::LOG_ERR) << "Opening serial port failed! Make sure cp210x kernel module is loaded, and /dev/ttyUSBxxx was created by cp210x!\n"
                         << "\tAlso make sure that /dev/ttyUSBxxx is R+W accessible by your user (usually enabled through udev.rules)!\n";
-    LOG(antpm::LOG_ERR) << "error=" << m_p->m_fd << ", strerror=" << ss << "\n";
+    LOG(antpm::LOG_ERR) << "error=" << m_p->m_fd << ", strerror=" << se << "\n";
     return rv;
   }
 
